<?php
/*

	Copyright (C) 2001 Richard D. Payne

	 This file is part of Teak (http://teak.sourceforge.net).

    Teak is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Teak is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Teak; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

Class TeakAddressBook extends GtkWindow {
	
	var $exit_button;
	var $address_books;
	
	function TeakAddressBook($text_entry) {
		$config;
			
		$this->GtkWindow();

		$this->set_title("Teak - Contact Lists");
		$this->set_usize (450,300);
	
		// This is for Drag and Drop
		$targets = array(array('text/plain', 0, -1));
			
		$this->realize();
			
		$this->load_address_book_configs ();
		
		$top_box = &new GtkVBox();
		$top_box->set_border_width(5);
		
		$toolbar = &new GtkHBox();
		$toolbar->set_spacing(5);
		
		$add_book_button = &new PictureButton("Add List","images/stock_book_red.xpm",$this,HORZ,"Add A New Contact List");
		$add_entry_button = &new PictureButton("Add Contact","images/stock_add.xpm",$this,HORZ,"Add A New Contact");
		$search_entry = &new GtkEntry();
		$search_button = &new PictureButton ("Search","images/stock_search.xpm",$this,HORZ,"Search With The Current List");
		$exit_button = &new PictureButton ("Exit","images/stock_exit.xpm",$this,HORZ,"Close The Address Book");
		
		$exit_button->connect('clicked',array(&$this,destroy_window));
		$this->exit_button = $exit_button;
		
		// This is incase the user presses the little 'x' from the window manager instead of our exit button
		$this->connect('delete-event',array(&$this,destroy_window));
		
		$toolbar->pack_start($add_book_button,FALSE,FALSE);
		$toolbar->pack_start($add_entry_button,FALSE,FALSE);
		$toolbar->pack_start($search_entry);
		$toolbar->pack_start($search_button,FALSE,FALSE);
		$toolbar->pack_end($exit_button,FALSE,FALSE);	
				
		$bottom_box = &new GtkHPaned();
		$bottom_box->set_border_width(5);
		
		$adb_list = &new GtkClist(2,array("Contact Lists","uid"));
		$adb_list->set_row_height(20);
		$adb_list->set_column_visibility(1,FALSE);
		$address_list = &new TeakAddressClist(4,array ("Name","Address","Phone","uid"));
		$address_list->set_column_visibility(3,FALSE);
		$address_list->set_column_width(0,100);
		$address_list->set_column_width(1,250);
		$address_list_scroller = &new GtkScrolledWindow();
		$address_list_scroller->add($address_list);
		
		$top_box->pack_start($toolbar,FALSE,FALSE);
		$top_box->pack_start($bottom_box);
		
		$bottom_box->add1($adb_list);
		$bottom_box->add2($address_list_scroller);
		
		$search_button->connect('clicked',array(&$this,search_for_entries),$search_entry,$adb_list,&$address_list);
		$add_book_button->connect('clicked',array(&$this, add_address_book),&$adb_list);
		
		$add_entry_button->connect('clicked',array(&$this, add_adb_entry),&$address_list,&$adb_list);
		
		$this->add($top_box);
		
	// Load the address bookd into the right hand list
		$this->load_address_books_into_list(&$adb_list);
		
	// Build the popup menu for the the address book list
		$address_book_popup=$this->build_popup_menu(&$adb_list);
		
	// Build the popup menu used for addresses
		$address_list_popup = $this->build_address_popup_menu(&$address_list,&$adb_list);	
		
		$adb_list->connect_object('button_press_event', array($this,show_address_book_popup_menu),$address_book_popup);
		$adb_list->connect("select-row",array(&$this,adb_selected),&$address_list);
		
		$address_list->connect_object('button_press_event', array($this,handle_address_list_mouse_events),$address_list_popup,&$text_entry,$address_list,$adb_list);
		
	// Setup drag and drop from the address_list to the address book list - this is the sending side
		$address_list->set_name("AddressList");
		$address_list->connect('drag_data_get', array(&$this,dnd_drag_data_get));
		$address_list->drag_source_set(GDK_BUTTON1_MASK|GDK_BUTTON3_MASK, $targets,
							GDK_ACTION_COPY);
							
	// Setup drag and drop from the address_list to the address book list - this is the receiving side
		$adb_list->set_name("AddressBookList");
		$adb_list->connect('drag_data_received',array(&$this,dnd_adb_received),$address_list);
		$adb_list->drag_dest_set(GTK_DEST_DEFAULT_ALL, $targets, GDK_ACTION_COPY);
		
		// If we have at least one contact list, select it
		if (count($this->address_books) > 0)
			$adb_list->select_row(0,0);
		
		$this->show();
		
	}
	// End Constructor
	
	/* ###################################################################################################################
		Gets called when the user drops something onto one of the address books in the clist
	################################################################################################################### */ 
	function dnd_drag_data_get ($widget, $context, $selection_data, $info, $time) {
		$dnd_sender = $widget->get_name();
		$selection_data->set($selection_data->target, 8, $dnd_sender);
	}
	
	/* ###################################################################################################################
		Gets called when the user drops something onto one of the address books in the clist
	################################################################################################################### */ 
	function dnd_adb_received ($widget, $context, $x, $y, $data, $info, $time,$address_list) {
		$address_books=$this->address_books;
	
		if ($data->data == "AddressList") {
			$position=$widget->get_selection_info($x,$y);
			
			//This means that wherever the user droped was invalid
			if ($position == FALSE)
				return;
			$book_uid = $widget->get_text($position[0],1);
			$book_number = $this->find_address_book($book_uid);
			
			//In case we get this far
			if ($address_books[$book_number]->type == "ldap")
				return;
		
			$this->copy_entry (null,$address_list,$address_books[$book_number]);
		}
	}
	
	/* ###################################################################################################################
		add an entry to the address book list
	################################################################################################################### */ 
	function find_address_book ($uid) {
		$address_books = $this->address_books;
		
		for ($i=0;$i<count($address_books);$i++) {
			if ($address_books[$i]->uid == $uid)
				return $i;
		}
		
		return NULL;
	}
	
	/* ###################################################################################################################
		add an entry to the address book
	################################################################################################################### */ 
	function add_adb_entry ($widget,$address_list,$adb_list) {
		
		$row = $adb_list->selection[0];
		if (!isset($row))
			return;
		
		// This gets us the uid (Unique ID) of the address book to which we'd like to add an entry
		$adb_uid = $adb_list->get_text($row,1);
		
		// Next we find that object in the address_books array of objects
		$address_books = $this->address_books;
		for ($i=0; $i < count($address_books); $i++) {
			if ($address_books[$i]->get_uid() == $adb_uid) {
				$selected_book = $address_books[$i];
				break;
				
			}
		}
		// We don't yet support adding to an ldap directory
		if ($selected_book->type == "ldap")
			return;
			
		if (($selected_book->type != tfile) && ($selected_book->conn == NULL)) {
		// This is a safety net, we should never get here w/o a connection, but still
			$success = $this->make_connection(&$selected_book);
			if (!$success) {
				$text = "Unable to connect to address book ".$selected_book->name.$success;
				$ed = &new error_dialog($text);
			}
		}
			
		$add_dialog = &new AddAddressBookEntryDialog("Add",&$selected_book);
		
		$add_dialog_add_button = $add_dialog->get_add_button();
		$add_dialog_add_button->connect('clicked',array(&$this,reload_single_address_book),$address_list,&$adb_list,$add_dialog);
		$add_dialog->show_all();
	}
		
	/* ###################################################################################################################
		Make a connection to the given address book based upon its type
	################################################################################################################### */ 
	function make_connection ($adb) {
		switch ($adb->type) {
			case db:
				if ($adb->password == "null") 
					$conn=mysql_connect("$adb->location","$adb->username");
				else
					$conn=mysql_connect("$adb->location","$adb->username","$adb->password");
	
				// Reassign the connect back into the object for later use
				$adb->conn=$conn;
			
				if (!isset($adb->conn))
					return mysql_error();
				else
					return TRUE;
			break;
			
			case ldap:
				$conn=ldap_connect($adb->get_location());
				$adb->conn=$conn;
				if (!isset($adb->conn))
					return ldap_error();
				else
					return TRUE;
			break;
		}
		
		
	}
	/* ###################################################################################################################
		add an address book to the config file
	################################################################################################################### */ 
	function add_address_book ($widget, $adb_list) {
		$add_dialog = &new AddAddressBookDialog("Add",null);
		$add_button = $add_dialog->get_add_button();
		$add_button->connect('clicked',array(&$this, add_button_pressed),&$adb_list,&$add_dialog);
		$add_dialog->show_all();
	}
	
	/* ###################################################################################################################
		Called when the add button of the dialog is pressed
			Reload the address books from the config file
			Reload the clist containing the lists
	################################################################################################################### */ 
	function add_button_pressed ($widget, $adb_list,$add_dialog) { 
		if ($add_dialog->get_success() == TRUE) {
			$add_dialog->destroy();
			$this->load_address_book_configs();
			$this->load_address_books_into_list(&$adb_list);
		}
		
	}
	
	/* ###################################################################################################################
		show the popup menu when the right mouse button is pressed on the address book list
	################################################################################################################### */ 
	function show_address_book_popup_menu ($event,$adb_popup) {
		if ($event->button == 3) {
			$adb_popup->popup(null,null,null, $event->button, $event->time);
		}
	}
	
	/* ###################################################################################################################
		handle any/all mouse events that occur in the address_list clist
	################################################################################################################### */ 
	function handle_address_list_mouse_events ($event,$address_popup,$text_entry,$address_list,$adb_list) {
		if ($event->button == 3) {
			$address_popup->popup(null,null,null, $event->button, $event->time);
		}
		
		// If the user double clicks on the address and we were passed a valid text_entry widget, add that users address to the widget
		elseif ($event->type == GDK_2BUTTON_PRESS) {
			// Add the address to the text entry in the compose window
			if ($text_entry !== NULL) {
				$row=$address_list->selection[0];
				if ($row === NULL)
					return;
				$address=$address_list->get_text($row,1);
				$current_text = $text_entry->get_chars(0,-1);
				if ($current_text == NULL)
					$text_entry->set_text($address);
				else {
					$new_text = $current_text.",".$address;
					$text_entry->set_text($new_text);
				}
			}
			// Else show the properties of the item
			else {
				if (isset($address_list->selection[0]))
					$this->change_address_entry_properties(NULL,$adb_list,$address_list);
			}
			
		}
			
	}
	
	/* ###################################################################################################################
		Change the properties of a address book item
	################################################################################################################### */ 
	function change_address_entry_properties($widget,$adb_list,$address_list) {
		$address_books=$this->address_books;
	
		//Find the address book that the user has selected
		$adb_selection = $adb_list->selection[0];
		$adb_uid = $adb_list->get_text($adb_selection,1);
		$adb_number = $this->find_address_book($adb_uid);
		
		//Now we find the entry that the user has selected, we want the uid
		$entry_selection = $address_list->selection[0];
		$entry_uid = $address_list->get_text($entry_selection,3);
		
		$change_dialog = &new AddAddressBookEntryDialog("Change",$address_books[$adb_number]);
		$apply_button = $change_dialog->get_add_button();
		$apply_button->connect('clicked',array(&$this, reload_single_address_book),&$address_list,&$adb_list,$change_dialog);
		$change_dialog->load_current_values($entry_uid);
		$change_dialog->show_all();
	}
	
	/* ###################################################################################################################
		Change the properties of a address book item
	################################################################################################################### */ 
	function reload_single_address_book ($widget,$address_list,$adb_list,$dialog) {
		$row=$adb_list->selection[0];
		$this->adb_selected ($adb_list, $row, null, null, $address_list);
		$dialog->destroy();
	}
	/* ###################################################################################################################
		build the popup menu that's displayed for each address in the list
	################################################################################################################### */ 
	function build_address_popup_menu($address_list,$adb_list) {
		$address_books = $this->address_books;
	
		$address_list_popup = &new GtkMenu();
		
		$compose_menu_item = &new GtkMenuItem("Compose To");
		$compose_menu_item->connect('activate',array($this,compose_message),$address_list);
		$address_list_popup->add($compose_menu_item);
		
		$sep_menu_item = &new GtkMenuItem();
		$address_list_popup->add($sep_menu_item);
		
		// Create another menu, this will be as a submenu that allows us to copy addresses from on book to another
		$copy_to_menu = &new GtkMenu();
		for ($i=0;$i<count($address_books);$i++) {
			if ($address_books[$i]->type != "ldap") {
			//Get the uid, the name is fine for the user, but we really care about the uid
				$mi_name=$address_books[$i]->name;
				$$mi_name=&new GtkMenuItem("$mi_name");
				$$mi_name->connect('activate',array(&$this,copy_entry),$address_list,&$address_books[$i]);
				$copy_to_menu->add($$mi_name);
			}
		}
		
		$copy_to_menu_item = &new GtkMenuItem("Copy To:");
		$copy_to_menu_item->set_submenu($copy_to_menu);
		
		$address_list_popup->add($copy_to_menu_item);
		
		$sep2_menu_item = &new GtkMenuItem();
		$address_list_popup->add($sep2_menu_item);
		
		$prop_menu_item = &new GtkMenuItem("Properties");
		 $prop_menu_item->connect('activate',array(&$this,change_address_entry_properties),$adb_list,$address_list);
		$address_list_popup->add($prop_menu_item);
		
		$del_menu_item = &new GtkMenuItem("Delete");
		$del_menu_item->connect('activate',array(&$this, delete_item_from_address_book),$adb_list,$address_list);
		$address_list_popup->add($del_menu_item);
		
		$address_list_popup->show_all();
		
		return $address_list_popup;
	}
	
	
/* ###################################################################################################################
	Delete a contact
################################################################################################################### */ 
	function delete_item_from_address_book($widget, $adb_list,$address_list) {
		$address_books = $this->address_books;
		
		//Find the address book that the user has selected
		$adb_selection = $adb_list->selection[0];
		$adb_uid = $adb_list->get_text($adb_selection,1);
		$adb_number = $this->find_address_book($adb_uid);
		
		//Now we find the entry that the user has selected, we want the uid
		$entry_selection = $address_list->selection[0];
		$entry_uid = $address_list->get_text($entry_selection,3);
		$address_list->unselect_row($entry_selection,0);
		
		$delete_entry = &new AddressBookEntry($entry_uid,null,null,null,null,null);
		$delete_entry->delete_entry($address_books[$adb_number]);
		
		$address_list->remove($entry_selection);
	
	}
	
/* ###################################################################################################################
	Copy an entry from one address book to another
################################################################################################################### */ 
	function copy_entry ($widget, $address_list,$copy_into_address_book) {
		$address_books = $this->address_books;
		
		// This is the selected row in the list of addresses
		$entry_row = $address_list->selection[0];
		
		// We don't yet support adding to an ldap directory
		if ($selected_book->type == "ldap")
			return;
		
		// Make sure we have a valid connection to the resource
		if (($copy_into_address_book->conn === NULL) && ($copy_into_address_book->type != tfile))
			$this->make_connection(&$copy_into_address_book);
				
		$name = $address_list->get_text($entry_row,0);
		
		// FIXME, this needs some work!
		// Here we try to make a First, MI and Last from the $name
		$num_spaces = substr_count($name," ");
		
		if ($num_spaces == 1) {
		
			$pos=strpos($name," ");
			$lname=substr($name,$pos+1);
			$fname=substr($name,0,$pos);
		
		}
		else {
		
			$lpos=strrpos($name," ");
			$lname=substr($name,$lpos+1);
		
			$fpos=strpos($name," ");
			$fname=substr($name,0,$fpos);
		
			$mi=substr($name,$fpos+1,1);
		}
		
		
		$email = $address_list->get_text($entry_row,1);
		$phone = $address_list->get_text($entry_row,2);
		
		$new_adb_entry_dialog = &new AddAddressBookEntryDialog ("Add",&$copy_into_address_book);
		$add_button = $new_adb_entry_dialog->get_add_button();
		$add_button->connect('clicked',array($this,destroy_dialog),$new_adb_entry_dialog);
		
		$new_adb_entry_dialog->set_text_field_values($fname,$mi,$lname,$phone,$email);
		$new_adb_entry_dialog->show_all();
		
		return;
	}
		
/* ###################################################################################################################
	Create a new compose window
################################################################################################################### */ 
	function compose_message ($menu_item, $address_list) {
		$selection = $address_list->selection[0];
		
		if ($selection === NULL)
			return;
		
		$address = $address_list->get_text($selection,1);
		$compose_window = &new Compose_Window ($address, null, null, null,null,null,null);
	
	}
	
	
/* ###################################################################################################################
	Build the popup menu for the address book list
################################################################################################################### */ 
	function build_popup_menu($adb_list) {
		$address_book_popup = &new GtkMenu();
		
		$add_menu_item = &new GtkMenuItem("Add Contact List");
		$address_book_popup->add($add_menu_item);
		$add_menu_item->connect('activate',array(&$this,add_address_book),&$adb_list);
		
		$delete_menu_item = &new GtkMenuItem("Delete");
		$address_book_popup->add($delete_menu_item);
		$delete_menu_item->connect('activate',array(&$this,delete_address_book),&$adb_list);
		
		$sep_menu_item = &new GtkMenuItem();
		$address_book_popup->add($sep_menu_item);
		
		$prop_menu_item = &new GtkMenuItem("Properties");
		$prop_menu_item->connect('activate',array(&$this,change_address_book),&$adb_list);
		$address_book_popup->add($prop_menu_item);
		
		$address_book_popup->show_all();
		
		return $address_book_popup;
	}
	
/* ###################################################################################################################
	Change the properties of an address book
################################################################################################################### */ 
	function change_address_book ($widget,$adb_list ) { 
	
		//Find the address book that the user has selected
		$adb_selection = $adb_list->selection[0];
		$adb_uid = $adb_list->get_text($adb_selection,1);
		$adb_number = $this->find_address_book($adb_uid);
		
		$change_dialog = &new AddAddressBookDialog("Change",&$adb_list);
		$apply_button = $change_dialog->get_add_button();
		$apply_button->connect('clicked',array(&$this, reload_address_books),&$adb_list,&$change_dialog);
		$change_dialog->load_current_values($this->address_books[$adb_number]);
		$change_dialog->show_all();
	}
/* ###################################################################################################################
		Reload the address books after a change has been made
###################################################################################################################*/
	function reload_address_books ($widget, $adb_list,$dialog) { 
		$this->load_address_book_configs();
		$this->load_address_books_into_list(&$adb_list);
		$dialog->destroy();
	}
	
/* ###################################################################################################################
	delete the selected address book
################################################################################################################### */ 
	function delete_address_book ($menu_item,$adb_list ) { 
		global $config;
		$selection = $adb_list->selection[0];
		$uid = $adb_list->get_text($selection,1);
		$delete_entry = &new AddressBookConfig($uid,null,null,null,null,null,null);
		$delete_entry->delete_address_book($config[preferences][address_book_config]);
		$adb_list->remove($selection);
		
	}
	
	/* ###################################################################################################################
		search for the entry in the chosen address book
	################################################################################################################### */ 
	function search_for_entries ($widget, $entry, $adb_list, $address_list ){ 
		
		$address_list->clear();
		$address_books=$this->address_books;
		
		if (($adb_list->selection[0]) === NULL)
			return;
		
		$adb_selection=$adb_list->selection[0];
		$adb_uid=$adb_list->get_text($adb_selection,1);
					
		for ($i=0;$i<count($address_books);$i++) {
			while (gtk::events_pending())
				gtk::main_iteration();
			if ($address_books[$i]->get_uid() == $adb_uid) {
				switch ($address_books[$i]->get_type()) {
					case tfile:
						//echo "Sorry, not yet implemented. \n";
					break;
					
					case ldap:
						$search_text=$entry->get_text();
						$search_string="(|(sn=*$search_text*)(givenname=*$search_text*))";
		
						if ($address_books[$i]->conn == NULL) 
							$this->make_connection(&$address_books[$i]);
						
						$binding=ldap_bind($address_books[$i]->conn);
						$sr=ldap_search($address_books[$i]->conn,"c=US", $search_string);  

						$info = ldap_get_entries($address_books[$i]->conn, $sr);
					
						for ($j=0; $j<($info["count"]); $j++) {
							$items=array($info[$j]["cn"][0],$info[$j]["mail"][0],null,null);
							$address_list->append(str_replace("\r","",$items));
						
						}
						return;
						
					break;
					
					case db:
						$search_text=$entry->get_text();
						
						if ($address_books[$i]->conn === NULL) 
							// We don't have a db connection yet
							$this->make_connection(&$address_books[$i]);
						
						if ($address_books[$i]->conn == FALSE)
							return;
						
						$table=$address_books[$i]->name;
						$db=$address_books[$i]->db;
						
						$query="SELECT * FROM $table WHERE fname like '$search_text' OR lname like '$search_text' or email like '$search_text'";
						$data=mysql_db_query("$db","$query",$address_books[$i]->conn);
						
						$address_list->freeze();
						$address_list->clear();
						
						while ($row_obj = mysql_fetch_object($data)) {
							$address_entry = &new AddressBookEntry ($row_obj->indx,$row_obj->fname, $row_obj->lname, $row_obj->mname,$row_obj->email,$row_obj->phone);
							$address_list->insert_entry_class($address_entry);	
						}
						
						$address_list->thaw();
					break;
				
				}
			}
		}
	
	}
	
	/* ###################################################################################################################
		Occurs when the user selected an address book from the left list
	################################################################################################################### */ 
	function adb_selected ($clist, $row, $col, $event, $address_list)  { 
		$address_books=$this->address_books;

		$address_list->freeze();
		$address_list->clear();
		
		
		list($adb_name,$spacing,$pixmap,$mask)=$clist->get_pixtext($row,0);
		$num_entries=0;
		
		for ($i=0;$i<count($address_books);$i++) {
			if ($address_books[$i]->get_name() == $adb_name) {
				switch ($address_books[$i]->get_type()) {
					case tfile:
						$file_name=$address_books[$i]->get_location();
						
					// Check the file exists before opening it
						if (file_exists($file_name) == FALSE) {
							echo "File doesn't exist. Bailing out\n";
							$address_list->thaw();
							return;
						}
						$fp = fopen ("$file_name","r");
						while (!feof ($fp)) {
						    $buffer = fgets($fp, 1000);						    
					    	    if ($buffer != NULL) {
							    list ($var, $value) = split("=",$buffer);
							    $$var=trim($value);
							     while ((($buffer2=fgets($fp,1000)) != "\n") && !feof($fp)) {
								    list ($var, $value) = split("=",$buffer2);
								    $$var=trim($value);
						            }
							    $address_entries[] = &new AddressBookEntry ($uid,$fn, $ln, $mi,$mail,$ph);
						    }
						}
						fclose ($fp);
						for ($i=0;$i<count($address_entries);$i++) 
							$address_list->insert_entry_class($address_entries[$i]);
							
						$address_list->thaw();
						return;
						break;
					
					case ldap:
						$address_list->thaw();
						return;
						break;
						
					case db;
						if ($address_books[$i]->conn === NULL) 
							// We don't have a db connection yet
							$this->make_connection(&$address_books[$i]);
						
													
						// Do a quick check before we go o, just in case
						if ($address_books[$i]->conn == FALSE)
							return;
							
						$query="SELECT * FROM ".$address_books[$i]->name;
						$data=mysql_db_query($address_books[$i]->db,"$query",$address_books[$i]->conn);
						
						while ($row_obj = mysql_fetch_object($data)) {
							$address_entry = &new AddressBookEntry ($row_obj->indx,$row_obj->fname, $row_obj->lname, $row_obj->mname,$row_obj->email,$row_obj->phone);
							$address_list->insert_entry_class($address_entry);	
						}
						
						$address_list->thaw();
						return;							
						break;
			        }


			}
			
		}
		
	}
	
	/* ###################################################################################################################
		Load the address book names into the clist on the left (aka, the list of address books)
	################################################################################################################### */ 
	function load_address_books_into_list ($clist)  { 

		$clist->freeze();
		$clist->clear();
		
		list($green_pixmap, $green_mask) = Gdk::pixmap_create_from_xpm($this->window,null,"images/stock_book_green.xpm");
		list($blue_pixmap, $blue_mask) = Gdk::pixmap_create_from_xpm($this->window,null,"images/stock_book_blue.xpm");
		list($yellow_pixmap, $yellow_mask) = Gdk::pixmap_create_from_xpm($this->window,null,"images/stock_book_yellow.xpm");
	
		$address_books=$this->address_books;
				
		for ($i=0; $i<count($address_books);$i++) {
			$row=$clist->append(array($address_books[$i]->get_name(),$address_books[$i]->get_uid()));
			switch ($address_books[$i]->get_type()) { 
				case ldap:
					$clist->set_pixtext($row,0,$address_books[$i]->get_name(),5,$green_pixmap,$green_mask);
				break;
				
				case tfile:
					$clist->set_pixtext($row,0,$address_books[$i]->get_name(),5,$blue_pixmap,$green_mask);
				break;
				
				case db:
					$clist->set_pixtext($row,0,$address_books[$i]->get_name(),5,$yellow_pixmap,$green_mask);
				break;
				
			}
		}
		$clist->thaw();		
	}
	
	
	/* ###################################################################################################################
		Load the address book config file, this tells us how many, what type, where etc....
	################################################################################################################### */ 
	function load_address_book_configs (){ 
		global $config;
		
		$this->address_books=NULL;	
					
		$fp = fopen($config[preferences][address_book_config],"r");
		$num_address_books=0;
		while ($buffer = fgetcsv ($fp, 2000)) {
			$num = count ($buffer);
			for ($c=0; $c<$num; $c++) {
				$temp=$buffer[$c];
				if (($temp[0] != "#") && ($temp[0] !== NULL)) {
					$address_books[] = &new AddressBookConfig($buffer[$c],$buffer[$c+1],$buffer[$c+2],$buffer[$c+3],$buffer[$c+4],$buffer[$c+5],$buffer[$c+6]);
					$num_address_books++;
					$c=$c+6;
				}
				else 
					$c=$c+6;
					
			}	    		    
		}
		
		fclose ($fp);
		$this->address_books=$address_books;	
      	}
	/* ###################################################################################################################
		return the exit button, to regiser for events
	################################################################################################################### */ 
	function get_exit_button() { 
		return $this->exit_button;		
	}
	
	/* ###################################################################################################################
		End gracefully by closing all the open connection first
	##################################################################################################*/
	function destroy_window($widget) {
		// First thing we need to do is close all open connections
		$address_books=$this->address_books;
		for ($i=0;$i<count($address_books);$i++) {
			if ($address_books[$i]->conn != NULL)
				switch ($address_books[$i]->type) {
				case db:
					mysql_close($address_books[$i]->conn);
				break;
				
				case ldap;
					ldap_close($address_books[$i]->conn);
				break;
				}
		}
		$this->destroy();
	}
	
	/* ###################################################################################################################
		If we can't connect to the server (be it db, ldap etc...) let the user now and handle it properly
	##################################################################################################*/
	
	function unable_to_connect ($type,$host) {
		$message="Unable to connect to $type server at $host\n";
		switch ($type) {
			case db:
			$message.=mysql_error();
			break;
		
			case ldap;
			$message.=ldap_error();
			break;
			}
		$ed = &new error_dialog($message);
			$ed->show();
	}
/* ###################################################################################################################
	Destroy the window we're passed
################################################################################################################### */ 
	function destroy_dialog ($widget, $dialog) {
		$dialog->destroy();
	}
}
