#!/usr/bin/php
<?php
/*

	Copyright (C) 2001 Richard D. Payne

	 This file is part of Teak (http://teak.sourceforge.net).

    Teak is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Teak is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Teak; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

	if (!class_exists('gtk')) {
		if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') {
			dl('php_gtk.dll');
			$OS=WIN;
		}
		else {
			dl('php_gtk.so');
			$OS=UNIX;
		}
	}

	//Setup a few constants, this is where teak itself is installed.
	define ("TEAKDIR",dirname(__FILE__)."/");
	define ("TEAKETCDIR",TEAKDIR."etc/");
	
	// FIXME Does this work in windows ??
	$user = getenv(USER);
	
	if ($OS == "WIN")
		$user_config_path = "c:\users\\".$user."\\.teak\\";
	else
		$user_config_path = "/home/".$user."/.teak/";
		
		
	// New user config code
	$config = load_config_file($user_config_path);
	$config[preferences][mime_file] = $user_config_path."mime.types";
	$config[preferences][address_book_config] = $user_config_path."adb.config";
	$config[preferences][temp_dir] = $user_config_path."temp/";
	
	$servers = load_servers($user_config_path);

	// This is the main part of teak, should soon be broken down into multiple files
	include (TEAKDIR."main_window.php");
	
	//Compose window
	include (TEAKDIR.'compose_class.php');

	// Address book specific indcludes
	include (TEAKDIR.'AddressBook/TeakAddressBookClass.php');
	include (TEAKDIR.'AddressBook/AdbConfigClass.php');
	include (TEAKDIR.'AddressBook/AdbEntryClass.php');
	include (TEAKDIR.'AddressBook/TeakAddressClist.php');
	include (TEAKDIR.'AddressBook/AddAddressBookDialog.php');
	include (TEAKDIR.'AddressBook/AddAddressBookEntryDialog.php');

	// Various dialogs
	include (TEAKDIR.'subscribe_class.php');
	include (TEAKDIR.'new_folder_dialog.php');
	include (TEAKDIR.'printer_dialog.php');
	include (TEAKDIR.'yes_no_dialog.php');
	include (TEAKDIR.'error_dialog.php');
	include (TEAKDIR.'password_dialog.php');
	
	//Utility widgets
	include (TEAKDIR.'widgets/PictureButton.php');
	include (TEAKDIR.'widgets/HeaderBlock.php');
	include (TEAKDIR.'widgets/FolderTree.php');
	include (TEAKDIR.'widgets/MessageList.php');
	include (TEAKDIR.'widgets/TeakText.php');


	//Library functions
	include (TEAKDIR.'lib/mime.php');
	
/* ####################################################################################################################
	Load the list of servers
####################################################################################################################*/
function load_servers($user_config_path) {
		$servers_file = $user_config_path."servers.ini";
		$servers = parse_ini_file($servers_file,TRUE);
		return $servers;
	}
	
/* ####################################################################################################################
	Load the configuration file(s) for us, if they don't exist, setup the defaults and reload
####################################################################################################################*/
function load_config_file($user_config_path) {
	global $OS;
	
	$config_file = $user_config_path."config.ini";
	
	// If the user's config file doesn't exist, then we copy in the defaults and call this again to load the default config
	if (!file_exists($config_file)) {
		new_user($user_config_path);
		$config = load_config_file($user_config_path);
		return $config;
	}
	else {
		$config = parse_ini_file("$config_file",TRUE);
		return $config;
	}
	
}

/* ####################################################################################################################
	Called when the user doesn't have the necessary file in their home directory, installs the defaults for us
####################################################################################################################*/
function new_user($user_config_path) {
	
	$files_to_copy = array ("mime.types","adb.config","config.ini","servers.ini");
	// Basic, should be a GUI wizard/druid etc...
	echo "You don't seem to have the necessary files for Teak installed in your home directory. \n";
	echo "I'm now going to make a new directory (.teak) and copy a few files in there\n";
	echo "(Press CTRL-C to abort)\n";
	sleep (5);
	
	echo "Making directory ";
	$success = mkdir ("$user_config_path",0700);
	if ($success)
		echo "done \n";
		
	echo "Making temp directory ";
	$success = mkdir ("$user_config_path/temp",0700);
	if ($success)
		echo "done \n";
		
	echo "Copying files ";
	foreach ($files_to_copy as $file) {
		$source=TEAKETCDIR.$file;
		$dest=$user_config_path.$file;
		copy ($source,$dest);
	}
	if ($success)
		echo "done \n";
		
	echo "\nTeak has now copied the sample config files into your configuration directory.\n";
	echo "Please edit the files ".$user_config_path."servers.ini and ".$user_config_path."config.ini \n";
	echo "and start teak again. \n\n";
	exit;
	
}
	
/*
 * Called when delete-event happens. Returns false to indicate that the event
 * should proceed.
 */
function delete_event()
{
	return false;
}

/*
 * Called when the window is being destroyed. Simply quit the main loop.
 */
function destroy($widget,$window)
{

	$window->close_imap_connection(null);
	Gtk::main_quit();
}


// Clean out the temp directory if the user wants us to (see config.ini)
//if ($config[preferences][clean_temp] == TRUE)
//	exec ("/bin/rm -rf $temp/* >/dev/null 2>&1 &");


$top_level_window = &new Main_Window();
$top_level_window->set_default_size(800,600);
$top_level_window->show();
$top_level_window->connect('destroy','destroy',array(null,$top_level_window));
$e_button=$top_level_window->get_exit_button();
$e_button->connect('clicked','destroy',$top_level_window);
$e_menu_item=$top_level_window->get_exit_menu_item();
$e_menu_item->connect('activate','destroy',$top_level_window);


/* Run the main loop. */
Gtk::main();


?>
